import typer
import requests
import math

from scrapse.leggitalia.utils import LeggiDiItalia, SECTIONS
from pathlib import Path
from bs4 import BeautifulSoup
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeRemainingColumn, TimeElapsedColumn
from concurrent.futures import ThreadPoolExecutor


def extract_file_name(soup, judgment_id):
    estrcomp = soup.find('div', class_='estrcomp')
    file_name = estrcomp.text.lower().split(
        [section.strip().lower() for section in SECTIONS if section in estrcomp.text][0], 1)[0].strip() + judgment_id
    file_name = file_name.replace(' ', '').replace('tribunale', '')
    return file_name


def search_judgments(ldi):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=False,
    ) as progress:
        progress.add_task(description="Searching for judgments...", total=None)
        request = requests.post(url=ldi.baseurl, headers=ldi.headers,
                                params=ldi.search_judgment_params)
    judgments_found = request.json()['result']['rows']
    progress.console.print(f'\u2713 Found {judgments_found} judgments')
    return typer.prompt('How many judgments do you want to download?', default=judgments_found, type=int)


def build_pagination(ldi, judgments_to_download):
    pagination = list()
    for index in range(0, math.ceil(judgments_to_download / ldi.batch_size), 1):
        start = index * ldi.batch_size
        rows = ldi.batch_size if (judgments_to_download - (
                (index + 1) * ldi.batch_size)) >= 0 else judgments_to_download - (ldi.batch_size * index)

        pagination.append((start, rows))
    # print(pagination)
    return pagination


def export_judgments_id(ldi, page):
    request = requests.post(url=ldi.baseurl, headers=ldi.headers, params=ldi.pagination_query(page))
    return request.json()['result']['items']


def export_judgments(ldi, items):
    judgments_id = [judgment['id'] for judgment in items]

    formatted_judgments_id = ['|'.join(judgments_id[i:i + ldi.batch_size]) for i in
                              range(0, len(judgments_id), ldi.batch_size)]

    request = requests.post(url=ldi.baseurl, headers=ldi.headers, params=ldi.build_export_query(formatted_judgments_id))

    soup = BeautifulSoup(request.text, 'html.parser')
    for index, judgment in enumerate(soup.find_all("div", {"class": "doc_body"})):
        output_file = Path('/'.join(
            (f'{ldi.judgment_directory_path}', f'{extract_file_name(judgment, judgments_id[index])}.{ldi.extension}')))
        output_file.write_text(str(judgment))


def scrap_judgments(
        judicial_bodies: str = typer.Option(None, "--judicial-bodies", "-j", help="Filter judging bodies"),
        location: str = typer.Option(None, "--location", "-l", help="Filter location of the court of reference"),
        sections: str = typer.Option(None, "--sections", "-s", help="Reference section filter, may change by location"),
        path: Path = typer.Option(LeggiDiItalia.main_directory_path, "--path", "-p",
                                  help="Path where to save downloaded judgments"),
        extension: str = typer.Option("HTM", "--extension", "-e", help="Desired extension of judgments"),
):
    """
        Extract the judgments from the platform https://pa.leggiditalia.it """'\n'"""
        It is necessary to specify at least one filter among: -j, -l and -s. """'\n'"""
        Provide an input list with the following formatting: -s 'par1, par2, ...'
    """
    ldi = LeggiDiItalia(judicial_bodies=judicial_bodies, location=location, sections=sections,
                        path=path, extension=extension, command='scrap_judgments')
    number_of_judgments = search_judgments(ldi)
    pagination = build_pagination(ldi, number_of_judgments)

    with Progress(
            "[progress.description]{task.description}",
            BarColumn(),
            "[progress.percentage]{task.percentage:>3.0f}%",
            TimeRemainingColumn(),
            TimeElapsedColumn(),
            refresh_per_second=5,
    ) as progress:
        futures = []
        overall_progress_task = progress.add_task(description="[green]Extraction of all judgments",
                                                  total=len(pagination))
        with ThreadPoolExecutor() as executor:
            for index, page in enumerate(pagination):
                judgments_id = export_judgments_id(ldi, page)
                futures.append(executor.submit(export_judgments, ldi, judgments_id))
                progress.update(overall_progress_task, completed=index)
            while (n_finished := sum([future.done() for future in futures])) < len(futures):
                progress.update(overall_progress_task, completed=n_finished + 1)



from scrapse.leggitalia import utils


def show_filters(j: bool = True, s: bool = True, l: bool = True):
    """
        Shows possible values to be assigned to respective filters to extract judgments
    """
    if j:
        print(f'Judicial bodies {utils.JUDICIAL_BODIES}\n')
    if s:
        print(f'Sections {utils.SECTIONS}\n')
    if l:
        print(f'Locations {utils.LOCATIONS}\n')



import typer
import requests
import json

from scrapse.leggitalia.utils import LeggiDiItalia
from rich.progress import Progress, SpinnerColumn, TextColumn


def extract_voices(ldi, key=None):
    request = requests.post(url=ldi.baseurl, headers=ldi.headers,
                            params=ldi.build_export_voices_query(key=key))
    voices = request.json()['result']['content']
    for voice in voices:
        if 'folder' in voice:
            del voice['folder']
            voice['items'] = extract_voices(ldi, key=voice['key'])
    return voices


def scrap_voices(
        path: str = typer.Option(LeggiDiItalia.main_directory_path, '--path', '-p',
                                 help='Path where to save json file'),
        show: bool = typer.Option(True, '--show', '-s', help='View extracted voices.')
):
    """
        Extraction of the voices tree
    """
    ldi = LeggiDiItalia(path=path, command='scrap_voices')
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=False,
    ) as progress:
        progress.add_task(description="Extraction of voices...", total=None)
        voices = json.dumps(extract_voices(ldi))
        json_file = open('/'.join([str(path), 'leggitalia_albero_voci.json']), 'w')
        json_file.write(voices)
        json_file.close()
    progress.console.print(f'\u2713 successfully extracted')



from pathlib import Path


def save_cookie(cookie: str):
    """
        Save session cookies in order to extract judgments
    """
    file_path = '/'.join([str(Path.cwd().absolute()), 'ldi_cookie.txt'])
    output_file = Path(file_path)
    output_file.parent.mkdir(parents=True, exist_ok=True)
    output_file.write_text(str(cookie))



import typer

from pathlib import Path
from rich.console import Console

LOCATIONS = ['asti', 'alessandria', 'cuneo', 'ivrea', 'novara', 'torino']

SECTIONS = [
    'Sez. Unite', '(Ad. Plen.)', 'Sez. I', 'Sez. I bis', 'Sez. I ter', 'Sez. I quater', 'Sez. II',
    'Sez. II bis', 'Sez. II ter', 'Sez. II quater', 'Sez. III', 'Sez. III bis', 'Sez. III ter',
    'Sez. III quater', 'Sez. IV', 'Sez. IV bis', 'Sez. IV ter', 'Sez. IV quater', 'Sez. V', 'Sez. V bis',
    'Sez. VI', 'Sez. VI - 1', 'Sez. VI - 2', 'Sez. VI - 3', 'Sez. VI - Lavoro', 'Sez. VI - 5', 'Sez. VII',
    'Sez. VIII', 'Sez. IX', 'Sez. X', 'Sez. XI', 'Sez. XII', 'Sez. XIII', 'Sez. XIV', 'Sez. XV', 'Sez. XVI',
    'Sez. XVII', 'Sez. XVIII', 'Sez. XIX', 'Sez. XX', 'Sez. XXI', 'Sez. XXII', 'Sez. XXIII', 'Sez. XXIV',
    'Sez. XXV', 'Sez. XXVI', 'Sez. XXVII', 'Sez. XXVIII', 'Sez. XXIX', 'Sez. XXX', 'Sez. XXXI', 'Sez. XXXII',
    'Sez. XXXIII', 'Sez. XXXIV', 'Sez. XXXV', 'Sez. XXXVI', 'Sez. XXXVII', 'Sez. XXXVIII', 'Sez. XXXIX',
    'Sez. XL', 'Sez. XLI', 'Sez. XLII', 'Sez. XLIII', 'Sez. XLIV', 'Sez. XLV', 'Sez. XLVI', 'Sez. XLVII',
    'Sez. XLVIII', 'Sez. XLIX', 'Sez. L', 'Sez. LI', 'Sez. LII', 'Sez. LIII', 'Sez. LIV', 'Sez. LV', 'Sez. LVI',
    'Sez. LVII', 'Sez. LVIII', 'Sez. LIX', 'Sez. LX', 'Sez. LXI', 'Sez. LXII', 'Sez. LXIII', 'Sez. LXIV',
    'Sez. LXV', 'Sez. LXVI', 'Sez. LXVII', '(Ad. Gen.)', 'Grande Sez.', 'Sez. App.', 'Sez. I App.',
    'Sez. II App.', 'Sez. III App.', 'Sez. agraria', 'Sez. atti norm.', 'Sez. Autonomie', 'Sez. comm. spec.',
    'Sez. consult.', 'Sez. contr.', 'Sez. contr. enti', 'Sez. Contr. gestione', "Sez. Contr. Legittimita'",
    'Sez. Contr. Stato', 'Sez. disciplinare', 'Sez. enti loc.', 'Sez. fall.', 'Sez. feriale', 'Sez. giurisdiz.',
    'Sez. lavoro', 'Sez. minori', 'Sez. riunite', 'Sez. Riunite cons.', 'Sez. Riunite contr.',
    'Sez. spec. in materia di imprese', 'Sez. spec. propr. industr. ed intell.', 'Sez. stralcio', 'II Stralcio',
    'Sez. uff. elettorale', 'Sez. Unica', 'Sez. I ampliata', 'Sez. II ampliata', 'Sez. III ampliata',
    'Sez. IV ampliata', 'Sez. V ampliata', 'VIII ampliata', 'Sez. VI ampliata', 'Sez. VII ampliata',
    'Grande Cam.', 'Sez. Spec. Immigr., Prot. Inter. e Libera circ. UE',
    'Sez. delle persone, dei minori e della fam.']

JUDICIAL_BODIES = [
    'Corte Costituzionale', 'Cassazione Civile', 'Cassazione Penale', 'Consiglio di Stato',
    'Collegio arbitrale', 'Corte di giustizia tributaria di primo grado',
    'Corte di giustizia tributaria di secondo grado', 'Commiss. Trib. I grado',
    'Commiss. Trib. II grado', 'Commissione Tributaria Centrale', 'Commissione tributaria provinciale',
    'Comm.Tributaria regionale', 'Commiss. usi civici', 'C.G.A. della Regione Siciliana',
    "Corte d'Appello", "Corte d'Appello militare", "Corte d'Assise", "Corte europea diritti dell'uomo",
    "Corte d'Assise d'Appello", 'Corte dei Conti', 'Corte di Giustizia CE',
    'Corte giustizia Unione Europea', "Autorita' Garante per la concorrenza", 'Giudice Istruttore',
    'Giudice di Pace', 'Giudice tutelare', 'T.A.R.', 'T.R.G.A.', 'Tribunale', "Trib. liberta'",
    'Tribunale Militare', 'Tribunale Minorenni', 'Trib. Reg. acque', 'Trib. Sorveglianza',
    'Tribunale Superiore delle Acque Pubbliche', 'Tribunale di Primo Grado CE',
    'Trib. I Grado Unione Europea', 'Uff. indagini preliminari']


class LeggiDiItalia:
    baseurl = 'https://pa.leggiditalia.it/rest'

    headers = {
        'Accept': 'application/json',
        'Content-type': 'application/x-www-form-urlencoded',
    }

    search_judgment_params = {
        'core': 'giuryPEL',
        'method': 'solr.search',
        'sort': 'docorder asc:titolo asc:elemorder asc:_is_parent desc',
        'facet_name': 'mask-sentenze',
        'reload': 'false',
        'fields': 'risultato:score:estrcomp',
        'score': 'true',
        'highlight': 'document_text',
        'filters': '{"mode":"AND","items":[{"field":"operasez","mode":"IN","value":["01","05","44","46","47","55",'
                   '"59","60","78","07","20","15","42","66","70","88","K0","K1","K3","K5","DA","DB","DC","DD","0H",'
                   '"2H","9H","90","41","43","H0","06","68","WT","WZ","86","QN","QP","Q6","W3","V3","WH","QE","WO",'
                   '"QK","WQ","VZ","WF","QC","W7","V7","WV","QO","XM","VD","WX","VV","WK","VW","WP","QL","WD","QA",'
                   '"WL","QH","WT","QN","X3","QS","86","Q6","XC","QZ","WM","QI","WS","QJ","WG","QD","XF","VI","WR",'
                   '"QM","X8","VA","WU","QW","X9","VB","WY","QX","WE","XA","VC","QB","XB","QY","X1","QR","WZ","QP",'
                   '"XU","VX","S7","Q7","AP"]}]}',
    }

    export_judgment_params = {
        'method': 'print.doc',
        'reload': '1',
        'mode': 'print',
        'FORMATO': 'HTM',
        'PAGINA': 'P',
        'NOMEEXPORT': 'Salva_documenti',
    }

    export_voices_params = {
        'fname': 'liste/class_sentenze.yaml',
        'method': 'utils.read_tree',
        'reload': 'false',
    }

    batch_size = 100

    judgment_directory_path = None

    voices_directory_path = None

    main_directory_path = '/'.join([str(Path.home()), 'scrapse', 'leggitalia'])

    def __init__(self, judicial_bodies=None, sections=None, location=None, extension=None, path=None,
                 command=None) -> None:
        super().__init__()
        if command == 'scrap_judgments':
            self.judicial_bodies = judicial_bodies
            self.sections = sections
            self.location = location
            self.extension = extension
            self.path = str(path)
            self.check_filters()
            self.build_headers()
            self.build_search_query()
            self.build_judgments_directory_path()
        elif command == 'scrap_voices':
            self.build_voices_directory_path()

    def check_filters(self):
        if self.judicial_bodies is None and self.sections is None and self.location is None:
            Console(stderr=True).print('No filter was provided.\nUse --help for more info.')
            raise typer.Exit()

    def build_headers(self):
        path = Path('ldi_cookie.txt')
        if path.is_file():
            self.headers['Cookie'] = path.open(mode='r').read()

    def build_judgments_directory_path(self):
        filters_to_join = [self.judicial_bodies, self.sections, self.location]
        directory = '_'.join(
            ['&'.join(filter.split(',')).replace(' ', '').lower() for filter in filters_to_join if filter is not None])

        self.judgment_directory_path = Path('/'.join((self.path, directory)))
        self.judgment_directory_path.mkdir(exist_ok=True, parents=True)

    def build_search_query(self):
        base_query = """{"mode":"AND","items":[{"mode":"AND","items":[{"field":"operasez","mode":"IN","value":["60","46","47","55","78","59","07"]},{"field":"tipo","mode":"EQUAL","value":"giurisprudenza/sentenze"}]}]}"""
        if self.judicial_bodies is not None:
            base_query = base_query[
                         :-2] + ''',{"field":"giuri_ente","mode":"IN","value":[''' + ','.join(
                f'"{item}"' for item in self.judicial_bodies.split(",")) + ''']}''' + base_query[
                                                                                      -2:]
        if self.sections is not None:
            base_query = base_query[
                         :-2] + ''',{"field":"giuri_sezione","mode":"IN","value":[''' + ','.join(
                f'"{item}"' for item in self.sections.split(",")) + ''']}''' + base_query[
                                                                               -2:]
        if self.location is not None:
            base_query = base_query[
                         :-2] + ''',{"field":"localita_ft","mode":"EQUAL","value":"''' + self.location + '''"}''' + base_query[
                                                                                                                    -2:]
        self.search_judgment_params['query'] = base_query

    def pagination_query(self, page):
        self.search_judgment_params['start'] = page[0]
        self.search_judgment_params['rows'] = page[1]
        return self.search_judgment_params

    def build_export_query(self, ids):
        self.export_judgment_params['ids'] = ids
        return self.export_judgment_params

    def build_voices_directory_path(self):
        self.judgment_directory_path = Path('/'.join([self.path, 'leggitalia_voices']))
        self.judgment_directory_path.mkdir(exist_ok=True, parents=True)

    def build_export_voices_query(self, key):
        self.export_voices_params['key'] = key
        return self.export_voices_params


class JudgmentMetadata:
    def __init__(self, nomefile=None, origine=None, tribunale=None, sezione=None, voci=None, sent_code=None,
                 sent_anno=None, nrg_code=None, nrg_anno=None, tipo=None) -> None:
        super().__init__()
        self.nomefile = nomefile
        self.origine = origine
        self.tribunale = tribunale
        self.sezione = sezione
        self.voci = voci
        self.sent_code = sent_code
        self.sent_anno = sent_anno
        self.nrg_code = nrg_code
        self.nrg_anno = nrg_anno
        self.tipo = tipo


class JudgmentCorpus:
    def __init__(self, nomefile=None, oggetto=None, fatto=None, decisione=None, fatto_decisione=None, pqm=None) -> None:
        super().__init__()
        self.nomefile = nomefile
        self.oggetto = oggetto
        self.fatto = fatto
        self.decisione = decisione
        self.fatto_decisione = fatto_decisione
        self.pqm = pqm



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scrapse', 'scrapse.leggitalia', 'scrapse.leggitalia.commands']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.2,<5.0.0',
 'requests>=2.28.2,<3.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['scrapse = scrapse.main:app']}

setup_kwargs = {
    'name': 'scrapse',
    'version': '0.3.7',
    'description': 'Package to download and manage judgments',
    'long_description': "# ScrapSE\n\n## Package description\n\nScrapSE downloads and manage the judgments.\n\nCurrently supported platforms: LEGGI D'ITALIA PA.\n\n### Install scrapse\n```\npip install scrapse\n```\nThe package creates the `scrapse` folder in `/Users/your_username`, where it will save all judgments in the  \nappropriate subfolders.\n\n### How to use\n\n#### Saving cookies - important!\n```\nscrapse leggitalia save-cookie 'your_cokies'\n```\nThis command saves session cookies in a special file, containing  `your_cookie`.\n\n#### Show filter values\n```\nscrapse leggitalia show-filters\n```\nThis command shows the possible values to be assigned to sentence search filters.\n\n#### Download the judgments\nMake sure you have **saved** platform-related cookies before downloading the judgments!.\n```\nscrapse leggitalia scrap-judgments -l torino -s 'Sez. lavoro, Sez. V'\n```\nThis command creates a folder in `/Users/your_username/scrapse/leggitalia` named `sez.lavoro&sez.v_torino` containing the judgments.\n\n#### Dump judgments to json format\n```\nscrapse leggitalia dump-judgments -d 'folder_path'\n```\nThis command creates the json files by saving them in the `/Users/your_username/scrapse/leggitalia/judgments_dump` folder.\n\n#### For more help\nFor more information for each command.\n```\ncommand-name --help\n```\n",
    'author': 'zaharia laurentiu jr marius',
    'author_email': 'zaharialorenzo@gmail.com',
    'maintainer': 'zaharia laurentiu jr marius',
    'maintainer_email': 'zaharialorenzo@gmail.com',
    'url': 'https://gitlab.di.unito.it/ngupp/ngupp-scrapse',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import json
import os
import typer
import re

from rich.progress import Progress, BarColumn, TimeRemainingColumn, TimeElapsedColumn
from datetime import datetime
from pathlib import Path
from concurrent.futures import ThreadPoolExecutor
from bs4 import BeautifulSoup
from scrapse.leggitalia.utils import LeggiDiItalia, JudgmentMetadata, JudgmentCorpus, SECTIONS


def read_file(path):
    # print(path)
    with open(path, 'r') as file:
        return BeautifulSoup(file, 'html.parser')


def extract_voices(soup):
    voices = None
    div_class = soup.find('div', class_='sent_classificazione')
    if div_class is not None:
        voices = list()
        split = list()
        spans = div_class.find_all('span', class_=True)
        for index, span in enumerate(spans[:-1]):
            if span['class'][0].split('_')[2] > spans[index + 1]['class'][0].split('_')[2]:
                split.append(index + 1)
            elif span['class'][0].split('_')[2] == 'l1' and spans[index + 1]['class'][0].split('_')[2] == 'l1':
                split.append(index + 1)

        split_spans = [[span for span in spans[s:e]] for s, e in zip([0] + split, split + [None])]
        for spans in split_spans:
            classification = ''
            if len(spans) > 1:
                for index, span in enumerate(spans[:-1]):
                    if span['class'] < spans[index + 1]['class']:
                        classification += span.text + '/' if (index < len(spans[:-1]) - 1) \
                            else span.text + '/' + spans[index + 1].text
                    elif span['class'] == spans[index + 1]['class']:
                        classification += span.text + '_' if (index < len(spans[:-1]) - 1) \
                            else span.text + '_' + spans[index + 1].text
            else:
                classification += spans[0].text
            voices.append(classification)
            # print(voices)
    return voices


def extract_object(soup):
    keyword = 'oggetto:'
    tag = soup.find(text=lambda t: t and keyword in t.lower())
    return tag.parent.text.replace('(asterisco)', '').lower().split(keyword, 1)[1].strip() if tag is not None else None


def extract_tribunal_section(soup):
    estrcomp = soup.find('div', class_='estrcomp')
    section = [section.strip().lower() for section in SECTIONS if section in estrcomp.text][0]
    tribunal = estrcomp.text.lower().split(section, 1)[0].replace('tribunale', '').strip()
    # print(f'tribunale {tribunal}')
    # print(f'section {section}')
    return tribunal, section


def extract_sent_anno(soup):
    text = soup.find('div', class_='estrcomp').text
    match_date = re.search(r'\d{2}/\d{2}/\d{4}', text)
    return datetime.strptime(match_date.group(), '%d/%m/%Y').year


def extract_metadata(soup, file_name):
    extract_sent_anno(soup)
    tribunale, sezione = extract_tribunal_section(soup)
    judgment_metadata = JudgmentMetadata(
        nomefile=file_name,
        origine='onelegale',
        tribunale=tribunale,
        sezione=sezione,
        voci=extract_voices(soup),
        sent_anno=extract_sent_anno(soup),
        tipo='sentenza di merito'
    )
    return judgment_metadata


def extract_corpus(soup, file_name):
    corpus = dict()

    menu_items = [item['href'].replace('#', '') for item in
                  soup.find('div', class_='sent_menu').find_all('a', href=True)]

    current_key = None
    for element in soup.find('a', id=menu_items[0]).parent.next_elements:
        if element.name == 'a' and element['id'] in menu_items:
            if element['id'] == 'diritto':
                current_key = 'decisione'
            elif element['id'] == 'dispositivo':
                current_key = 'pqm'
            elif element['id'] == 'fatto-diritto':
                current_key = 'fatto-decisione'
            else:
                current_key = element['id']
            corpus[current_key] = list()
        elif element.name == 'p':
            corpus[current_key].append(element.text.strip())

    for key, value in corpus.items():
        if value is not None:
            corpus[key] = ' '.join(corpus[key])

    judgment_corpus = JudgmentCorpus(
        nomefile=file_name,
        oggetto=extract_object(soup),
        fatto=corpus.get('fatto'),
        decisione=corpus.get('decisione'),
        fatto_decisione=corpus.get('fatto-decisione'),
        pqm=corpus.get('pqm'),
    )

    return judgment_corpus


def extract_json(soup, file):
    file_name = file.split('/')[-1].split('.')[0]
    judgment_metadata = extract_metadata(soup, file_name)
    judgment_corpus = extract_corpus(soup, file_name)
    return judgment_metadata, judgment_corpus


def build_dict(file, metadata_dict, corpus_dict):
    metadata, corpus = extract_json(read_file(file), file)
    tribunale_key = metadata.tribunale.replace(' ', '_')
    sent_anno_key = str(metadata.sent_anno)

    if metadata_dict.get(tribunale_key) is None:
        metadata_dict[tribunale_key] = dict()
        corpus_dict[tribunale_key] = dict()
    if metadata_dict.get(tribunale_key).get(sent_anno_key) is None:
        metadata_dict[tribunale_key][sent_anno_key] = list()
        corpus_dict[tribunale_key][sent_anno_key] = list()

    metadata_dict[tribunale_key][sent_anno_key].append(metadata)
    corpus_dict[tribunale_key][sent_anno_key].append(corpus)


def save_json(dict, path, directory):
    path.joinpath(directory).mkdir(exist_ok=True, parents=True)
    for tribunale_key in dict.keys():
        for sent_anno_key, value in dict[tribunale_key].items():
            full_path = '/'.join([str(path), directory, f'{tribunale_key}_{sent_anno_key}.json'])
            with open(full_path, 'w') as f:
                f.write(json.dumps([metadata.__dict__ for metadata in value], indent=4, ensure_ascii=False))


def dump_judgments(
        directory: str = typer.Option(None, '--directory', '-d', help='Folder path from which to read judgments.',
                                      show_default=False),
        file: str = typer.Option(None, '--file', '-f', help='Path to the file to read.', show_default=False),
        extension: str = typer.Option('HTM', '--extension', '-e', help='File extension to read.'),
        path: str = typer.Option('/'.join([LeggiDiItalia.main_directory_path, 'judgments_dump']), '--path', '-p',
                                 help='Folder path from which to read judgments.'),
):
    """
    Dump judgments to json format
    """
    with Progress(
            "[progress.description]{task.description}",
            BarColumn(),
            "[progress.percentage]{task.percentage:>3.0f}%",
            TimeRemainingColumn(),
            TimeElapsedColumn(),
            refresh_per_second=5,
    ) as progress:
        dump_judgments_directory_path = Path(path)
        dump_judgments_directory_path.mkdir(exist_ok=True, parents=True)

        files_to_read = ['/'.join([directory, filename]) for filename in os.listdir(directory) if
                         filename.endswith(extension)] if directory is not None else [file]
        metadata_dict, corpus_dict = dict(), dict()

        progress_task = progress.add_task(description="[green]Dump progress", total=len(files_to_read))

        for index, file in enumerate(files_to_read):
            build_dict(file, metadata_dict, corpus_dict)
            progress.update(progress_task, completed=index)

        with ThreadPoolExecutor() as executor:
            executor.submit(save_json, metadata_dict, dump_judgments_directory_path, 'metadata')
            executor.submit(save_json, corpus_dict, dump_judgments_directory_path, 'corpus')



import typer

from pathlib import Path
from scrapse.leggitalia.commands import scrap_judgments
from scrapse.leggitalia.commands import dump_judgments
from scrapse.leggitalia.commands import scrap_voices
from scrapse.leggitalia.commands import save_cookie
from scrapse.leggitalia.commands import show_filters

leggitalia_app = typer.Typer()
leggitalia_app.command()(scrap_judgments.scrap_judgments)
leggitalia_app.command()(dump_judgments.dump_judgments)
leggitalia_app.command()(scrap_voices.scrap_voices)
leggitalia_app.command()(save_cookie.save_cookie)
leggitalia_app.command()(show_filters.show_filters)


@leggitalia_app.command()
def show_voices():
    """
    Shows the tree of voices
    """
    pass


@leggitalia_app.callback()
def callback():
    """Dedicated command to the site LEGGI D'ITALIA PA"""
    main_directory_path = Path('/'.join([str(Path.home()), 'scrapse', 'leggitalia']))
    main_directory_path.mkdir(exist_ok=True, parents=True)


if __name__ == "__main__":
    leggitalia_app()
